@@ -1,5 +1,5 @@
 // Copyright (C) 2020 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com, author Marc Mutz <marc.mutz@kdab.com>
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GFDL-1.3-no-invariants-only
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qanystringview.h"
 #include "qdebug.h"
@@ -129,7 +129,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \fn template <typename Char> QAnyStringView::QAnyStringView(const Char *str, qsizetype len)
+    \fn template <typename Char, QAnyStringView::if_compatible_char<Char> = true> QAnyStringView::QAnyStringView(const Char *str, qsizetype len)
 
     Constructs a string view on \a str with length \a len.
 
@@ -146,7 +146,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \fn template <typename Char> QAnyStringView::QAnyStringView(const Char *first, const Char *last)
+    \fn template <typename Char, QAnyStringView::if_compatible_char<Char> = true> QAnyStringView::QAnyStringView(const Char *first, const Char *last)
 
     Constructs a string view on \a first with length (\a last - \a first).
 
@@ -221,7 +221,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \fn template <typename Container, if_compatible_container<Container>> QAnyStringView::QAnyStringView(const Container &str)
+    \fn template <typename Container, QAnyStringView::if_compatible_container<Container>> QAnyStringView::QAnyStringView(const Container &str)
 
     Constructs a string view on \a str. The length is taken from \c{std::size(str)}.
 
@@ -440,8 +440,10 @@ QT_BEGIN_NAMESPACE
     Returns a string view containing \a n code points of this string view,
     starting at position \a pos.
 
+//! [UB-sliced-index-length]
     \note The behavior is undefined when \a pos < 0, \a n < 0,
     or \a pos + \a n > size().
+//! [UB-sliced-index-length]
 
     \sa first(), last(), chopped(), chop(), truncate(), {Sizes and Sub-Strings}
 */
@@ -453,7 +455,9 @@ QT_BEGIN_NAMESPACE
     Returns a string view starting at position \a pos in this object,
     and extending to its end.
 
+//! [UB-sliced-index-only]
     \note The behavior is undefined when \a pos < 0 or \a pos > size().
+//! [UB-sliced-index-only]
 
     \sa first(), last(), chopped(), chop(), truncate(), {Sizes and Sub-Strings}
 */
@@ -567,7 +571,9 @@ QT_BEGIN_NAMESPACE
 /*!
     \fn QAnyStringView::compare(QAnyStringView lhs, QAnyStringView rhs, Qt::CaseSensitivity cs)
 
-    Returns an integer that compares to zero as \a lhs compares to \a rhs.
+    Compares the string view \a lhs with the string view \a rhs and returns a
+    negative integer if \a lhs is less than \a rhs, a positive integer if it is
+    greater than \a rhs, and zero if they are equal.
 
     If \a cs is Qt::CaseSensitive (the default), the comparison is case sensitive;
     otherwise the comparison is case-insensitive.
