@@ -80,9 +80,9 @@ QWasmWindow::QWasmWindow(QWindow *w, QWasmDeadKeySupport *deadKeySupport,
 
     QWasmClipboard::installEventHandlers(m_canvas);
 
-    // set inputmode to none to stop mobile keyboard opening
+    // set inputMode to none to stop mobile keyboard opening
     // when user clicks anywhere on the canvas.
-    m_canvas.set("inputmode", std::string("none"));
+    m_canvas.set("inputMode", std::string("none"));
 
     // Hide the canvas from screen readers.
     m_canvas.call<void>("setAttribute", std::string("aria-hidden"), std::string("true"));
@@ -115,12 +115,6 @@ QWasmWindow::QWasmWindow(QWindow *w, QWasmDeadKeySupport *deadKeySupport,
     m_pointerLeaveCallback =
             std::make_unique<qstdweb::EventCallback>(m_qtWindow, "pointerleave", pointerCallback);
 
-    m_dropCallback = std::make_unique<qstdweb::EventCallback>(
-            m_qtWindow, "drop", [this](emscripten::val event) {
-                if (processDrop(*DragEvent::fromWeb(event)))
-                    event.call<void>("preventDefault");
-            });
-
     m_wheelEventCallback = std::make_unique<qstdweb::EventCallback>(
             m_qtWindow, "wheel", [this](emscripten::val event) {
                 if (processWheel(*WheelEvent::fromWeb(event)))
@@ -133,9 +127,20 @@ QWasmWindow::QWasmWindow(QWindow *w, QWasmDeadKeySupport *deadKeySupport,
         event.call<void>("stopPropagation");
     });
 
+   emscripten::val keyFocusWindow;
+    if (QWasmIntegration::get()->inputContext()) {
+        QWasmInputContext *wasmContext =
+            static_cast<QWasmInputContext *>(QWasmIntegration::get()->inputContext());
+        // if there is an touchscreen input context,
+        // use that window for key input
+        keyFocusWindow = wasmContext->m_inputElement;
+    } else {
+        keyFocusWindow = m_qtWindow;
+    }
+
     m_keyDownCallback =
-            std::make_unique<qstdweb::EventCallback>(m_qtWindow, "keydown", keyCallback);
-    m_keyUpCallback = std::make_unique<qstdweb::EventCallback>(m_qtWindow, "keyup", keyCallback);
+            std::make_unique<qstdweb::EventCallback>(keyFocusWindow, "keydown", keyCallback);
+    m_keyUpCallback = std::make_unique<qstdweb::EventCallback>(keyFocusWindow, "keyup", keyCallback);
 
     setParent(parent());
 }
@@ -158,6 +163,11 @@ QSurfaceFormat QWasmWindow::format() const
     return window()->requestedFormat();
 }
 
+QWasmWindow *QWasmWindow::fromWindow(QWindow *window)
+{
+    return static_cast<QWasmWindow *>(window->handle());
+}
+
 void QWasmWindow::onRestoreClicked()
 {
     window()->setWindowState(Qt::WindowNoState);
@@ -188,7 +198,7 @@ void QWasmWindow::onNonClientAreaInteraction()
 bool QWasmWindow::onNonClientEvent(const PointerEvent &event)
 {
     QPointF pointInScreen = platformScreen()->mapFromLocal(
-            dom::mapPoint(event.target, platformScreen()->element(), event.localPoint));
+        dom::mapPoint(event.target(), platformScreen()->element(), event.localPoint));
     return QWindowSystemInterface::handleMouseEvent(
             window(), QWasmIntegration::getTimestamp(), window()->mapFromGlobal(pointInScreen),
             pointInScreen, event.mouseButtons, event.mouseButton,
@@ -362,6 +372,8 @@ void QWasmWindow::raise()
 {
     bringToTop();
     invalidate();
+    if (QWasmIntegration::get()->inputContext())
+        m_canvas.call<void>("focus");
 }
 
 void QWasmWindow::lower()
@@ -377,12 +389,8 @@ WId QWasmWindow::winId() const
 
 void QWasmWindow::propagateSizeHints()
 {
-    QRect rect = windowGeometry();
-    if (rect.size().width() < windowMinimumSize().width()
-        && rect.size().height() < windowMinimumSize().height()) {
-        rect.setSize(windowMinimumSize());
-        setGeometry(rect);
-    }
+    // setGeometry() will take care of minimum and maximum size constraints
+    setGeometry(windowGeometry());
     m_nonClientArea->propagateSizeHints();
 }
 
@@ -517,13 +525,13 @@ bool QWasmWindow::processKey(const KeyEvent &event)
 
 bool QWasmWindow::processPointer(const PointerEvent &event)
 {
-    if (event.pointerType != PointerType::Mouse)
+    if (event.pointerType != PointerType::Mouse && event.pointerType != PointerType::Pen)
         return false;
 
     switch (event.type) {
     case EventType::PointerEnter: {
         const auto pointInScreen = platformScreen()->mapFromLocal(
-                dom::mapPoint(event.target, platformScreen()->element(), event.localPoint));
+            dom::mapPoint(event.target(), platformScreen()->element(), event.localPoint));
         QWindowSystemInterface::handleEnterEvent(
                 window(), m_window->mapFromGlobal(pointInScreen), pointInScreen);
         break;
@@ -538,30 +546,6 @@ bool QWasmWindow::processPointer(const PointerEvent &event)
     return false;
 }
 
-bool QWasmWindow::processDrop(const DragEvent &event)
-{
-    m_dropDataReadCancellationFlag = qstdweb::readDataTransfer(
-            event.dataTransfer,
-            [](QByteArray fileContent) {
-                QImage image;
-                image.loadFromData(fileContent, nullptr);
-                return image;
-            },
-            [this, event](std::unique_ptr<QMimeData> data) {
-                QWindowSystemInterface::handleDrag(window(), data.get(),
-                                                   event.pointInPage.toPoint(), event.dropAction,
-                                                   event.mouseButton, event.modifiers);
-
-                QWindowSystemInterface::handleDrop(window(), data.get(),
-                                                   event.pointInPage.toPoint(), event.dropAction,
-                                                   event.mouseButton, event.modifiers);
-
-                QWindowSystemInterface::handleDrag(window(), nullptr, QPoint(), Qt::IgnoreAction,
-                                                   {}, {});
-            });
-    return true;
-}
-
 bool QWasmWindow::processWheel(const WheelEvent &event)
 {
     // Web scroll deltas are inverted from Qt deltas - negate.
@@ -577,7 +561,7 @@ bool QWasmWindow::processWheel(const WheelEvent &event)
     })();
 
     const auto pointInScreen = platformScreen()->mapFromLocal(
-            dom::mapPoint(event.target, platformScreen()->element(), event.localPoint));
+        dom::mapPoint(event.target(), platformScreen()->element(), event.localPoint));
 
     return QWindowSystemInterface::handleWheelEvent(
             window(), QWasmIntegration::getTimestamp(), window()->mapFromGlobal(pointInScreen),
