@@ -1,6 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // Copyright (C) 2016 Intel Corporation.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QtCore/QCoreApplication>
@@ -89,6 +89,7 @@ private slots:
     void qDebugQFlags() const;
     void qDebugStdChrono_data() const;
     void qDebugStdChrono() const;
+    void qDebugStdOptional() const;
     void textStreamModifiers() const;
     void resetFormat() const;
     void defaultMessagehandler() const;
@@ -1215,6 +1216,28 @@ void tst_QDebug::qDebugStdChrono() const
     QCOMPARE(fn(), expected);
 }
 
+void tst_QDebug::qDebugStdOptional() const
+{
+    QString file, function;
+    int line = 0;
+    MessageHandlerSetter mhs(myMessageHandler);
+    {
+        std::optional<QByteArray> notSet = std::nullopt;
+        std::optional<QByteArray> set("foo");
+        auto no = std::nullopt;
+        QDebug d = qDebug();
+        d << notSet << set << no;
+    }
+#ifndef QT_NO_MESSAGELOGCONTEXT
+    file = __FILE__; line = __LINE__ - 4; function = Q_FUNC_INFO;
+#endif
+    QCOMPARE(s_msgType, QtDebugMsg);
+    QCOMPARE(s_msg, QString::fromLatin1("nullopt std::optional(\"foo\") nullopt"));
+    QCOMPARE(QString::fromLatin1(s_file), file);
+    QCOMPARE(s_line, line);
+    QCOMPARE(QString::fromLatin1(s_function), function);
+}
+
 void tst_QDebug::textStreamModifiers() const
 {
     QString file, function;
