@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
@@ -472,59 +472,47 @@ void tst_QMessageBox::staticSourceCompat()
     // source compat tests for < 4.2
 QT_WARNING_PUSH
 QT_WARNING_DISABLE_DEPRECATED
+#define COMPARE(real, exp) do {\
+    const auto pressed = static_cast<QMessageBox::StandardButton>(real);\
+    const auto expected = static_cast<QMessageBox::StandardButton>(exp);\
+    if (!QTest::qCompare(pressed, expected, #real, #exp, __FILE__, __LINE__)) \
+        return; } while (false)
+
     ExecCloseHelper closeHelper;
     closeHelper.start(Qt::Key_Enter);
     ret = QMessageBox::information(nullptr, "title", "text", QMessageBox::Yes, QMessageBox::No);
-    int expectedButton = int(QMessageBox::Yes);
-    if (const QPlatformTheme *theme = QGuiApplicationPrivate::platformTheme()) {
-        const int dialogButtonBoxLayout = theme->themeHint(QPlatformTheme::DialogButtonBoxLayout).toInt();
-        if (dialogButtonBoxLayout == QDialogButtonBox::MacLayout
-            || dialogButtonBoxLayout == QDialogButtonBox::GnomeLayout)
-            expectedButton = int(QMessageBox::No);
-    }
-    QCOMPARE(ret, expectedButton);
+    COMPARE(ret, QMessageBox::No);
     QVERIFY(closeHelper.done());
 
     closeHelper.start(Qt::Key_Enter);
     ret = QMessageBox::information(nullptr, "title", "text", QMessageBox::Yes | QMessageBox::Default, QMessageBox::No);
-    QCOMPARE(ret, int(QMessageBox::Yes));
+    COMPARE(ret, int(QMessageBox::Yes));
     QVERIFY(closeHelper.done());
 
 #if QT_DEPRECATED_SINCE(6, 2)
     // The overloads below are valid only before 6.2
     closeHelper.start(Qt::Key_Enter);
     ret = QMessageBox::information(nullptr, "title", "text", QMessageBox::Yes, QMessageBox::No | QMessageBox::Default);
-    QCOMPARE(ret, int(QMessageBox::No));
+    COMPARE(ret, int(QMessageBox::No));
     QVERIFY(closeHelper.done());
 
     closeHelper.start(Qt::Key_Enter);
     ret = QMessageBox::information(nullptr, "title", "text", QMessageBox::Yes | QMessageBox::Default, QMessageBox::No | QMessageBox::Escape);
-    QCOMPARE(ret, int(QMessageBox::Yes));
+    COMPARE(ret, int(QMessageBox::Yes));
     QVERIFY(closeHelper.done());
 
     closeHelper.start(Qt::Key_Enter);
     ret = QMessageBox::information(nullptr, "title", "text", QMessageBox::Yes | QMessageBox::Escape, QMessageBox::No | QMessageBox::Default);
-    QCOMPARE(ret, int(QMessageBox::No));
+    COMPARE(ret, int(QMessageBox::No));
     QVERIFY(closeHelper.done());
 
     // the button text versions
     closeHelper.start(Qt::Key_Enter);
     ret = QMessageBox::information(nullptr, "title", "text", "Yes", "No", QString(), 1);
-    QCOMPARE(ret, 1);
+    COMPARE(ret, 1);
     QVERIFY(closeHelper.done());
-
-    if (0) { // don't run these tests since the dialog won't close!
-        closeHelper.start(Qt::Key_Escape);
-        ret = QMessageBox::information(nullptr, "title", "text", "Yes", "No", QString(), 1);
-        QCOMPARE(ret, -1);
-        QVERIFY(closeHelper.done());
-
-        closeHelper.start(Qt::Key_Escape);
-        ret = QMessageBox::information(nullptr, "title", "text", "Yes", "No", QString(), 0, 1);
-        QCOMPARE(ret, 1);
-        QVERIFY(closeHelper.done());
-    }
 #endif // QT_DEPRECATED_SINCE(6, 2)
+#undef COMPARE
 QT_WARNING_POP
 }
 
@@ -808,7 +796,7 @@ void tst_QMessageBox::hideNativeByDestruction()
     // Make it application modal so that we don't end up with a sheet on macOS
     dialog->setWindowModality(Qt::ApplicationModal);
     window.show();
-    QVERIFY(QTest::qWaitForWindowActive(&window));
+    QVERIFY(QTest::qWaitForWindowExposed(&window));
     dialog->open();
 
     // We test that the dialog opens and closes by watching the activation of the
